#####################################################################
# 
# ZScheduler -- Zope Scheduled Event Product
# Copyright (C) 2000 iCompression, Inc.
#
# Based on code by Martijn Pieters, (c) 1999 Antraciet B.V.
# Used with explicit permission.
# 
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License  
# along with this program; if not, write to the 
#     Free Software Foundation, Inc., 
#     59 Temple Place - Suite 330, 
#     Boston, MA  02111-1307, USA.
# 
#####################################################################
""" ZSchedule Event Catalog """

import Globals
from OFS.Folder import Folder
from Products.ZCatalog import ZCatalog
from Products.ZCatalog import Catalog
from DateTime import DateTime
from Threading import Event

# The one instance of ZSchedule has an ID known to all ZEvents
ScheduleID='Schedule' # The well-known ID for the Schedule
ScheduleChange=Event() # Global communication between Schedule and Dispatcher

class ZSchedule(ZCatalog.ZCatalog):
  """ An event Schedule. This is also a container. """

  meta_type = 'ZSchedule'
  icon='misc_/ZScheduler/ZScheduleIcon'

  manage_options = (
    {'label':'Contents', 'action':'manage_main'},
    {'label':'Cataloged Objects', 
      'action': 'manage_catalogView', 'target': 'manage_main'},
    {'label':'Import/Export', 'action':'manage_importExportForm'},
    {'label':'Security', 'action':'manage_access'},
    {'label':'Undo', 'action':'manage_UndoForm'},
  )

  __ac_permissions__ = (
    ('View management screens', 
      ('manage_main','manage_menu','manage_catalogView')),
    ('Access contents information',
      ('objectIds', 'objectValues', 'objectItems',''),
      ('Anonymous', 'Manager')),
    ('Delete objects', ('manage_delObjects',)),
    ('Import/Export objects', (
      'manage_importObject','manage_importExportForm',
      'manage_exportObject')),
  )

  def __init__(self, id='Schedule', title='ZEvent Catalog'):
    self.id = id
    self.title = title
    self.threshold = 1000
    self._v_total = 0
    self._catalog = ZCatalog.Catalog()

    self._catalog.addColumn('id')
    self._catalog.addColumn('title')
    self._catalog.addColumn('nextEventTime')
    self._catalog.addColumn('absolute_url')
    self._catalog.addIndex('nextEventTime', 'FieldIndex')

  # override ZCatalog methods
  def catalog_object(self, obj, uid):
    """ override ZCatalog to set ScheduleChange event """
    ZCatalog.ZCatalog.catalog_object.im_func(self, obj, uid)
    ScheduleChange.set()
      
  def uncatalog_object(self, uid):
    """ override ZCatalog to set ScheduleChange event """
    try: ZCatalog.ZCatalog.uncatalog_object.im_func(self, uid)
    except ValueError: pass
    ScheduleChange.set()

  # canned ZCatalog queries of the Schedule store:
  def armed_event(self):
    """ get the first event that is armed and ready """
    next_url=None
    next_time=None
    for event in self._catalog.searchResults(
            nextEventTime=DateTime(1),
            nextEventTime_usage='range:min',
            sort_on='nextEventTime'
            ):
      next_url = event.absolute_url
      next_time = event.nextEventTime
      break  # just look at the first one
    return (next_time, next_url)

Globals.default__class_init__(ZSchedule)
