#####################################################################
# 
# ZScheduler -- Zope Scheduled Event Product
# Copyright (C) 2000 iCompression, Inc.
#
# Based on code by Martijn Pieters, (c) 1999 Antraciet B.V.
# Used with explicit permission.
# 
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License  
# along with this program; if not, write to the 
#     Free Software Foundation, Inc., 
#     59 Temple Place - Suite 330, 
#     Boston, MA  02111-1307, USA.
# 
#####################################################################
""" Zope Scheduler (ZScheduler) Product """

import ZSchedule
import ZEvent       

from Globals import ImageFile, DateTime
from Threading import Thread
from ZPublisher import Client

# The well-known ID for the Schedule
ScheduleID=ZSchedule.ScheduleID 
# Global communication between Schedule and Dispatcher
ScheduleChange=ZSchedule.ScheduleChange 

import zLOG
import sys, traceback, string
#import pdb; pdb.set_trace()

def loggerr(severity, summary, detail='', error=None, reraise=None):
  if severity > 199:
    type, val, tb = sys.exc_info()
    fe=traceback.format_exception (type, val, tb) 
    zLOG.LOG(__name__, severity, summary, string.join(fe, ''), 
      (type, val, tb), reraise)
    del type, val, tb
  else:
    zLOG.LOG(__name__, severity, summary, detail, None, reraise)

# We only want one instance of the ZSchedule class,
# therefore we're not using registerClass
# Because no registerClass, 
# use the old method to set the icon for the ZSchedule object
misc_ = { 'ZScheduleIcon': ImageFile('www/ZSchedule.gif', globals()) }

def initialize(context):
  try:
    # Add the schedule catalog to the Zope root, 
    app = context._ProductContext__app

    global Schedule
    if hasattr(app, ScheduleID):
      #loggerr(0, 'Found existing '+ScheduleID)
      Schedule = getattr(app, ScheduleID, None)
    else:
      loggerr(0, 'Starting to create '+ScheduleID)
      try: # creating Schedule index
        oSchedule=ZSchedule.ZSchedule(id=ScheduleID, title='ZEvent Catalog')
        app._setObject(ScheduleID, oSchedule)
        loggerr(0, 'Created new '+ScheduleID)
      except: # if you can't create the index, tell someone.
        loggerr(200, 'Failed to create new '+ScheduleID)
      Schedule = getattr(app, ScheduleID, None)

    assert Schedule is not None

    # initialize ZEvent objects
    #loggerr(0, 'Starting to register event class.')
    try: 
      context.registerClass(
        ZEvent.OneTimeZEvent,
        permission = 'Add ZEvent',
        constructors = (
          ZEvent.manage_addOneTimeZEventForm,
          ZEvent.manage_addOneTimeZEvent),
        icon = 'www/OneTimeZEvent.gif')
    except: # if you can't register the class, tell someone.
      loggerr(200, 'Failed to register ZEvent class.')
      import sys, traceback, string
      type, val, tb = sys.exc_info()
      sys.stderr.write(string.join(
        traceback.format_exception(type, val, tb), ''))
      del type, val, tb    

    # Start running the Dispatcher
    dispatcher=Thread(target=Timer, name='ZEventTimer')
    dispatcher.start()

  except:
    import traceback
    traceback.print_exc()

#The Dispatcher runs as a separate thread
# It uses the Schedule as its primary data structure
# It knows which event is next
# It sleeps until that event or a change in the Schedule
# It fires an event and logs its output

maxwait=float(5)  # max time to wait between wake-ups in seconds

def Timer():     # aka Dispatcher
  # infinite loop
  while 1:
    # Good morning. 
    # The first thing we need is some coffee and a new connection.
    import Zope
    app=Zope.app()
    try: 
      Schedule = getattr(app, ScheduleID, None)
    except:
      loggerr(300, 'Cannot access catalog. Suspending operation.')
      break
    while 1:
      interval=maxwait # Default sleep time. May be recalculated below
      try: 
        (atime, aurl)=Schedule.armed_event() # Get next armed event
      except:
        loggerr(300,'Cannot access catalog. Suspending operation.')
        break
      if atime is None: 
        #loggerr(0, 'Awakened with no armed events.')
        break     # Sleep some more
      elif DateTime.isFuture(atime): # The next armed event is not yet ready
        #   calculate how long we have to wait
        ainterval=atime.timeTime() - DateTime().timeTime()
        if ainterval < float(0): ainterval=float(0) # Is negative bad? 
        #emsg= 'Next ready event: %s, %s, %s' % (atime, aurl, ainterval)
        #loggerr(0, emsg)
        interval=ainterval # Comment out for debugging to limit to maxwait 
        break              # Sleep some more
      else:                        # This event is ready now.
        # fire event, and log its output
        furl=string.join((aurl,'trigger'),'/') # URL for triggering event
        #furl=aurl # Debug
        emsg= '\nTrigger event: %s\nTrigger time: %s' % (aurl, atime)
        try:
          (headers, response)=Client.call(furl,None,None) # Fire event 
          #dmsg=string.join((headers, response),'\n')
          dmsg='%s\n' % response
          loggerr(0, emsg, detail=dmsg) # Log the event and its output
        except:
          # Failed to trigger the event. 
          #### Bug. What keeps us from trying to fire it again? Nothing.
          type, val, tb = sys.exc_info()
          dmsg="Failed to run event's action method.\nType=%s\nVal=%s\n" % (type, val)
          loggerr(100, emsg, detail=dmsg)
          del type, val, tb
          try: Client.call('%s/%s' % (aurl, 'disarm')) # Attempt to disarm
          except: pass # This event will come back to haunt us
        app._p_jar.sync() 
        continue  # Go get next event without sleeping
        #interval=2 # Wait so we don't get stuck in tight loop during debug
        #break 

    # End of inner while 1 loop
    #  We're going to sleep now; so, free the connection
    app._p_jar.close()
    del app
    #  Sleep for predetermined interval
    ScheduleChange.wait(interval) # in seconds (float)

    if ScheduleChange.isSet():
      ScheduleChange.clear()
      # Schedule has changed, we woke up early
      # Loop back to the top and check for
      # an earlier event than we were waiting for
      pass
    else:
      # We timed out, so there must be a ready event
      # Loop back to the top and trigger the next ready event. 
      # That's probably the one we were waiting for.
      pass

  # End of outer while 1 loop.
  # Something bad happened, let's clean up before quitting for good
  try:
    app._p_jar.close()
    del app
  except:
    pass
  loggerr(100, 'Dispatcher thread is terminating.')

# end of module